# $Copyright: Copyright (c) 2010 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#-----------------------------------------------------------------------------
# $Header: /project/isv-agent/ACCLib/lib/vcs/Resource.pm,v 1.16 2009/07/27 11:59:51 aatre Exp $
#
# Veritas Software Corporation
#
# $Author: aatre $
# $Date: 2009/07/27 11:59:51 $
# $Revision: 1.16 $
#
# Provides: package Resource
#
# Functionality: VCS Resource Related Commands
#
# Authors: 011787
#-----------------------------------------------------------------------------

#-------------------
# version calls.
#-------------------
use Version;
Version::TellVersion("\$Header: /project/isv-agent/ACCLib/lib/vcs/Resource.pm,v 1.16 2009/07/27 11:59:51 aatre Exp $::ETVHeader");
#-----------------------
# end of version calls.
#-----------------------

#------------------------
# Declare package name.
#------------------------
package Resource;

#----------------------------------------
# Turn on strict for subs, vars and refs.
#----------------------------------------
use strict;

#--------------------------------------
# Include the package name required.
#--------------------------------------
use LogInt;
use Hares;
use VCSType; #so we can get the timeouts for online/offline


=pod

-----------------------------------------------------------------------------
Function: Boolean Online( IN String ResourceName, 
                          IN String SystemName )

Description:
  Onlines the given resource on specified node/system.
 
Parameters:
  ResourceName : Name of the resource.
  SystemName   : Name of the system/node on which the resource needs to bring online.
	
Return Value:
  Boolean value in the set of {0,1} where
         0 - false
         1 - true
-----------------------------------------------------------------------------

=cut

sub Online {

  my ( $sResourceName, 
       $sSystemName ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10460,
                   "Resource",
                   "Online",
                   "Subroutine %s called with: Resource [%s], System [%s]",
                   $sMyName, $sResourceName, $sSystemName );

  if(Hares::Online($sResourceName, $sSystemName)) {
    #all ok - contine
  }
  else {
    LogInt::Message( LogInt::LLError,
                     10461,
                     "Resource",
                     "Online",
                     "failed in call to Hares::Online");
    return 0; #error return
  }

  my $iOnlineCheckPeriod   = 10; #period to wait between checks
  my $iOnlineMaxWaitPeriod = undef;
  my $sResourceType        = undef;

  #--------------------------------------
  # first get the type of this resource.
  #--------------------------------------
  if(Resource::GetTypeNameGivenResource($sResourceName, \$sResourceType)) {
    #-------------------------------------------
    # all ok continue
    # get the online timeout for this type
    #-------------------------------------------
    if(VCSType::GetOnlineTimeoutGivenType($sResourceType, \$iOnlineMaxWaitPeriod)) {
      #all ok continue
      LogInt::Message( LogInt::LLTrace,
                       10462,
                       "Resource",
                       "Online",
                       "call to VCSType::GetOnlineTimeoutGivenType got [$iOnlineMaxWaitPeriod]");
    } 
    else {
      LogInt::Message( LogInt::LLError,
                       10463,
                       "Resource",
                       "Online",
                       "failed in call to VCSType::GetOnlineTimeoutGivenType");
      return 0; #error return
    }
  } 
  else {
    LogInt::Message( LogInt::LLError,
                     10464,
                     "Resource",
                     "Online",
                     "failed in call to Resource::GetTypeNameGivenResource");
    return 0; #error return
  }

  #-------------------------------------------------------------------------
  # commands ran ok, need to wait for the resource to come online
  # get state waiting for the output to be 'ONLINE'
  # or the output to contain 'FAULTED' or for the maxwaitperiod to expire.
  #-------------------------------------------------------------------------
  my $iNumberOfCheckPeriodsWaited = 0;
  my $iReturnState                = 0;

  while(($iOnlineCheckPeriod * $iNumberOfCheckPeriodsWaited) < $iOnlineMaxWaitPeriod) {
    my $sResourceState = 0;
    if(Hares::GetStateInfoGivenResourceAndSystem($sResourceName, $sSystemName, \$sResourceState)) {
      if($sResourceState eq "ONLINE") {
        LogInt::Message( LogInt::LLTraceTrace,
                         10465,
                         "Resource",
                         "Online",
                         "Resource [$sResourceName] is online, exiting with success" );
        $iReturnState = 1; #set good state
        last; #jump loop
      } 
      elsif($sResourceState eq "FAULTED") {
        LogInt::Message( LogInt::LLError,
                         10466,
                         "Resource",
                         "Online",
                         "Resource [$sResourceName] is faulted; failed to online");
        $iReturnState=0; #set error state
        last; #jump loop
      } 
      else {
        my $sMessage = "Resource:[$sResourceName], System:[$sSystemName], waiting for online - currently state:[$sResourceState]";
        LogInt::Message( LogInt::LLTraceTrace,
                         10467,
                         "Resource",
                         "Online",
                         $sMessage );
        sleep $iOnlineCheckPeriod;
        $iNumberOfCheckPeriodsWaited++;
      }

    }
    else {
      #--------------------------------------------
      # failed to get the state of the resource.
      #--------------------------------------------
      LogInt::Message( LogInt::LLError,
                       10468,
                       "Resource",
                       "Online",
                       "failed in call to Hares::GetStateInfoGivenResourceAndSystem");
      $iReturnState = 0; #set error state
      last; #jump loop
    }
  } #end while waiting for online

  return $iReturnState;
}


=pod

-----------------------------------------------------------------------------
Function: Boolean GetParentResourceListGivenChild(
                                    IN String ChildResource,
                                    OUT ArrayOfStrings ParentResourceList )

Description:
  Gives the parent resources for a given child.

Parameters:
  ChildResource      : Name of the child resource for which parents are needed.
  ParentResourceList : List of parent resources for the give child resource.

Return Value:
  Boolean value in the set of {0,1} where
          0 - false
          1 - true
-----------------------------------------------------------------------------

=cut

sub GetParentResourceListGivenChild {

  my ( $sChildResource, 
       $rlsParentResourceList ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10469,
                   "Resource",
                   "GetParentResourceListGivenChild",
                   "Subroutine %s called with: Child Resource Name [%s]",
                   $sMyName, $sChildResource );

  my @lsHaresDepOutput = ();

  if(Hares::GetDependInfoListGivenResource($sChildResource, \@lsHaresDepOutput)) {
    #call worked ok
  } 
  else {
    LogInt::Message( LogInt::LLError,
                     10470,
                     "Resource",
                     "GetParentResourceListGivenChild",
                     "failed in call to Hares::GetDependInfoListGivenResource");
    return 0; #exit function with error
  }

  #-----------------
  # spin the list.
  #-----------------

  for my $sEachDependEntry (@lsHaresDepOutput) {
    #--------------------------------------------------------------------
    # split the line (somthin like  SG2 samba_share2 samba_server2).
    #--------------------------------------------------------------------
    my ( $sTmpGroup,
         $sTmpParent,
         $sTmpChild ) = split(/\s+/, $sEachDependEntry);

    LogInt::Message( LogInt::LLTraceTrace,
                     10471,
                     "Resource",
                     "GetParentResourceListGivenChild",
                     "Group [$sTmpGroup], parent [$sTmpParent], child [$sTmpChild]");
    #-----------------------------------------------------------
    # add to new list where the child is the passed in child.
    #-----------------------------------------------------------
    if($sChildResource eq $sTmpChild) {
      push(@$rlsParentResourceList, $sTmpParent);
      LogInt::Message( LogInt::LLTraceTrace,
                       10472,
                       "Resource",
                       "GetParentResourceListGivenChild",
                       "added parent [$sTmpParent]" );
    }
  }

  return 1; #exit function - good status
}


=pod

-----------------------------------------------------------------------------
Function: Boolean Offline( IN String ResourceName, 
                           IN String SystemName )

Description:
  Brings the resource offline on the specified node/system.

Parameters:
  ResourceName : Name of the resource to bring offline.
  SystemName   : Name of the node/system on which the resource
                 needs brought  offline.

Return Value:
  Boolean value in the set of {0,1} where
        0 - false
        1 - true
-----------------------------------------------------------------------------

=cut

sub Offline {

  my ( $sResourceName, 
       $sSystemName ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10473,
                   "Resource",
                   "Offline",
                   "Subroutine %s called with: Resource [%s], System [%s]",
                   $sMyName, $sResourceName, $sSystemName );

  my $sResourceState = undef;
  if(Hares::GetStateInfoGivenResourceAndSystem( $sResourceName, 
                                                    $sSystemName, 
                                                    \$sResourceState )) {
    if($sResourceState eq "ONLINE") {
      LogInt::Message( LogInt::LLTrace,
                       10474,
                       "Resource",
                       "Offline",
                       "Resource [$sResourceName] is online" );
      #--------------------------------------------------------------
      # continue this might have parents that need to be put offline
      #--------------------------------------------------------------
    }
    else {
      #-----------------------------------------------------------------
      # if we're offline then we know all our parents cant be online.
      #-----------------------------------------------------------------
      LogInt::Message( LogInt::LLTrace,
                       10475,
                       "Resource",
                       "Offline",
                       "Resource [$sResourceName] is NOT online, exit good");
      return 1; #exit good status
    }

  } 
  else {
    #------------------------------------
    # call to get resource state failed.
    #------------------------------------
    LogInt::Message( LogInt::LLError,
                     10476,
                     "Resource",
                     "Offline",
                     "Failed in call to GetStateInfoGivenResourceAndSystem" );
    return 0; #exit function with error
  }

  #--------------------------------------------------------------------------
  # for this online resource determine if it has parents,
  # call ourselves for each parent
  # after calling ourselves for each parent we can put this resource offline
  # if error we need to abort asap and return the error
  #--------------------------------------------------------------------------
  my @lsParentResourceList = ();
  my $iReturnCode          = undef;

  $iReturnCode = GetParentResourceListGivenChild($sResourceName, \@lsParentResourceList);

  if($iReturnCode) {
    #-----------------------------
    # see if we have any parents
    #-----------------------------
    for my $sEachParentResource (@lsParentResourceList) {
      #--------------------------------------------
      # for each parent call ourselves (recurse)
      #--------------------------------------------
      $iReturnCode = Offline($sEachParentResource, $sSystemName);

      #-------------------------------------------------
      # if we get a error then we should get out now
      #-------------------------------------------------
      if($iReturnCode) {
        #all ok continue
      } 
      else {
        last; #error of some sort - jump loop
      }
    } #end for each parent loop

    #-------------------------------------------------
    # if things are ok then put this resource offline
    #-------------------------------------------------
    if($iReturnCode) {
      #offline this resource now
      LogInt::Message( LogInt::LLTrace, 
                       10477,
                       "Resource",
                       "Offline",
                       "Going to OFFLINE resource [$sResourceName] on system [$sSystemName]");
      $iReturnCode = OfflineThisResourceItemWaitInternal($sResourceName, $sSystemName);
    }
  }

  return $iReturnCode;
}


=pod

-----------------------------------------------------------------------------
Function: Boolean OfflineThisResourceItemWaitInternal( IN String ResourceName,
                                                       IN String SystemName )

Description:
  Offlines the given resource and waits internally until the
  resource is not ONLINE.

Parameters:
  ResourceName : Name of the resource.
  SystemName   : Name of the node/system on which the resource 
                 needs to bring offline.

Return Value:
  Boolean value in the set of {0,1} where
        0 - false
        1 - true
-----------------------------------------------------------------------------

=cut

sub OfflineThisResourceItemWaitInternal {

  my ( $sResourceName,
       $sSystemName ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10478,
                   "Resource",
                   "OfflineThisResourceItemWaitInterval",
                   "Subroutine %s called with: Resource Name [%s], System Name [%s]",
                   $sMyName, $sResourceName, $sSystemName );

  #------------------------
  # offline this resource
  #------------------------
  LogInt::Message( LogInt::LLTrace,
                   10479,
                   "Resource",
                   "OfflineThisResourceItemWaitInterval",
                   "going to OFFLINE resource [$sResourceName] on system [$sSystemName]");

  if(Hares::Offline($sResourceName, $sSystemName)) {
    #call to offline ok
  }
  else {
    #---------------------------------------------------------------
    # offline request failed - print error and return error code
    #---------------------------------------------------------------
    LogInt::Message( LogInt::LLError,
                     10480,
                     "Resource",
                     "OfflineThisResourceItemWaitInterval",
                     "failed in call to Hares::Offline");
    return 0; #return error
  }

  my $iOfflineCheckPeriod    = 10; #period to wait between checks
  my $iOfflineMaxWaitPeriod  = undef;
  my $sResourceType          = undef;

  #--------------------------------------
  # first get the type of this resource
  #--------------------------------------
  if(Resource::GetTypeNameGivenResource($sResourceName, \$sResourceType)) {
    #------------------------------------------
    # all ok continue
    # get the offline timeout for this type
    #------------------------------------------
    if(VCSType::GetOfflineTimeoutGivenType($sResourceType, \$iOfflineMaxWaitPeriod)) {
      # all ok continue
      LogInt::Message( LogInt::LLTrace, 
                       10481,
                       "Resource",
                       "OfflineThisResourceItemWaitInterval",
                       "call to VCSType::GetOfflineTimeoutGivenType got [$iOfflineMaxWaitPeriod]");
    } 
    else {
      LogInt::Message( LogInt::LLError, 
                       10482,
                       "Resource",
                       "OfflineThisResourceItemWaitInterval",
                       "failed in call to VCSType::GetOfflineTimeoutGivenType");
      return 0; #error return
    }
  } 
  else {
    LogInt::Message( LogInt::LLError,
                     10483,
                     "Resource",
                     "OfflineThisResourceItemWaitInterval",
                     "failed in call to Resource::GetTypeNameGivenResource");
    return 0; #error return
  }

  my $iNumberOfCheckPeriodsWaited = 0;
  my $iReturnState                = 0;

  while(($iOfflineCheckPeriod * $iNumberOfCheckPeriodsWaited) < $iOfflineMaxWaitPeriod) {
    my $sResourceState = undef;
    if(Hares::GetStateInfoGivenResourceAndSystem( $sResourceName, 
                                                      $sSystemName,
                                                      \$sResourceState )) {
      if($sResourceState eq "ONLINE") {
        LogInt::Message( LogInt::LLTraceTrace,
                         10484,
                         "Resource",
                         "OfflineThisResourceItemWaitInterval",
                         "Resource [$sResourceName] is online, waiting for not online" );
        #continue waiting
      } 
      else {
        LogInt::Message( LogInt::LLTraceTrace,
                         10485,
                         "Resource",
                         "OfflineThisResourceItemWaitInterval",
                         "Resource [$sResourceName] is NOT online, exit good");
        return 1;
      }

    } 
    else {
      #----------------------------
      # call to get state failed
      #----------------------------
      LogInt::Message( LogInt::LLError,
                       10486,
                       "Resource",
                       "OfflineThisResourceItemWaitInterval",
                       "Failed in call to Hares::GetStateInfoGivenResourceAndSystem");
      return 0; #return error
    }
    sleep $iOfflineCheckPeriod;
    $iNumberOfCheckPeriodsWaited++;
  } #end while resource is online

  LogInt::Message( LogInt::LLTraceTrace,
                   10487,
                   "Resource",
                   "OfflineThisResourceItemWaitInterval",
                   "Resource [$sResourceName] didn't good offline waited [$iOfflineMaxWaitPeriod]");

  return 0; #resource never went offline
}


=pod

-----------------------------------------------------------------------------
Function: Boolean GetGroupNameGivenResource( IN String ResourceName, 
                                             OUT String ServiceGroupName )

Description:
  Gives the resource group for a given resource.

Parameters:
  ResourceName     : Name of the resource.
  ServiceGroupName : Name of the service group for which the resource belongs.

Return Value:
  Boolean value in the set of {0,1) where
          0 - false
          1 - true
-----------------------------------------------------------------------------

=cut

sub GetGroupNameGivenResource {

  my ( $sResourceName,
       $rsServiceGroupName ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10488,
                   "Resource",
                   "GetGroupNameGivenResource",
                   "Subroutine %s callled with: Resource Name [%s], Group Name [%s]",
                   $sMyName, $sResourceName, $rsServiceGroupName );

  LogInt::Message( LogInt::LLTrace,
                   10489,
                   "Resource",
                   "GetGroupNameGivenResource",
                   "Calling with resource [$sResourceName], calling Hares::GetGroupNameGivenResource");

  #-------------------------------------------------------------
  # construct the call to the generic Hares Display command.
  #-------------------------------------------------------------

  my $sResources  = $sResourceName;
  my $sAttributes = "Group"; 
  my $sGroups     = undef;
  my $sTypes      = undef;
  my $sSystems    = undef;
  my @lsResults   = ();

  if(Hares::Display($sResources, $sAttributes, $sGroups, $sTypes, $sSystems, \@lsResults)) {
    # return ok - continue
  }
  else {
    #-----------------------------------
    # call to get display info failed
    #-----------------------------------
    LogInt::Message( LogInt::LLError,
                     10490,
                     "Resource",
                     "GetGroupNameGivenResource",
                     "Failed in call to Hares::Display" );
    return 0; #return error
  }

  my ( $sResourceTrash,
       $sAttributeTrash,
       $sSystemTrash, 
       $sValue ) = split(/\s/, $lsResults[0]);

  LogInt::Message( LogInt::LLTrace,
                   10491,
                   "Resource",
                   "GetGroupNameGivenResource",
                   "Called with resource [$sResourceName], returning GroupName:[$sValue]");

  $$rsServiceGroupName = $sValue;

  return 1;
}


=pod

-----------------------------------------------------------------------------
Function: Boolean GetTypeNameGivenResource( IN String ResourceName, 
                                            OUT String ResourceType )

Description:
  Gives the type name of the given resource.

Parameters:
  ResourceName : Name of the resource name.
  ResourceType : Type of the resource given.

Return Value:
  Boolean value in the set of {0,1} where
        0 - false
        1 - true
-----------------------------------------------------------------------------

=cut

sub GetTypeNameGivenResource {

  my ( $sResourceName,
       $rsTypeName ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10492,
                   "Resource",
                   "GetTypeNameGivenResource",
                   "Subroutine %s called with: Resource Name [%s]",
                   $sMyName, $sResourceName );

  #---------------------------------------------------------------
  # construct the call to the generic Hares Display command
  #---------------------------------------------------------------
  my $sResources  = $sResourceName;
  my $sAttributes = "Type"; 
  my $sGroups     = undef;
  my $sTypes      = undef;
  my $sSystems    = undef;

  my @lsResults   = ();

  if(Hares::Display($sResources, $sAttributes, $sGroups, $sTypes, $sSystems, \@lsResults)) {
    # return ok - continue
  } 
  else {
    #-----------------------------------
    # call to get display info failed
    #-----------------------------------
    LogInt::Message( LogInt::LLError,
                     10493,
                     "Resource",
                     "GetTypeNameGivenResource",
                     "Failed in call to Hares::Display" );
    return 0; #return error
  }

  my ( $sResourceTrash, 
       $sAttributeTrash,
       $sSystemTrash, 
       $sValue ) = split(/\s/, $lsResults[0]);

  LogInt::Message( LogInt::LLTrace,
                   10494,
                   "Resource",
                   "GetTypeNameGivenResource",
                   "Called with resource [$sResourceName], returning TypeName:[$sValue]");

  #spagare VCSOne change
  $sValue =~ s/\(.*//g;
  $$rsTypeName = $sValue;

  return 1;
}
 

=pod

-----------------------------------------------------------------------------
Function: Boolean hares( IN String HaCmd,
                         IN String ResourceName,
                         IN String SystemName )

Description:
  A rapper to the hares command.

Parameters:
  HaCmd      : Command which describes the one of the following actions:
                 Online
                 Offline
  SystemName : Name of the node/system in the cluster on which the
               group needs to bring Onbline or offline.
  WaitPeriod : Number of seconds to wait before the specified action completes.

Return Value:
  Boolean value in the set of {0, 1} where
        0 - false
        1 - true
-----------------------------------------------------------------------------

=cut

sub hares {

  my ( $sHaCmd,
       $sResourceName,
       $sSystemName ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10495,
                   "Resource",
                   "hares",
                   "Subroutine %s called with: HA command [%s], Resource Name [%s], System Name [%s]",
                   $sMyName, $sHaCmd, $sResourceName, $sSystemName );

  if ($sHaCmd eq "online") {
    return Resource::Online($sResourceName, $sSystemName);
  }
  elsif ($sHaCmd eq "offline") {
    return Resource::Offline($sResourceName, $sSystemName);
  }
  else {
    #---------------------------------
    # Unrecognised command argument.
    #---------------------------------
    LogInt::Message( LogInt::LLTrace,
                     10496,
                     "Resource",
                     "hares",
                     "Received invalid HaCmd string [%s]", $sHaCmd );
    return 0;
  }
}


1;

